unit fmProcessList;

// Copyrigths (C) 2000 Alexey A. Dynnikov <aldyn@chat.ru>

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TProcessListForm = class(TForm)
    LV: TListView;
    BnRefresh: TButton;
    Timer: TTimer;
    procedure BnRefreshClick(Sender: TObject);
    procedure LVColumnClick(Sender: TObject; Column: TListColumn);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    SortIdx: Integer;       // The index of column to be sorted
    Reversed: Boolean;      // Sort in reverse order
    Refreshing: Boolean;    // I am refreshing, do not disturb!
    procedure Resort;
  public
    { Public declarations }
  end;

var
  ProcessListForm: TProcessListForm;

implementation

{$R *.DFM}

uses SvCom_ProcessList;

procedure TProcessListForm.BnRefreshClick(Sender: TObject);
var i: Integer;
    SL: TStringList;
    LI: TListItem;
    SPID: String;
    PID: Integer;
begin
    if Refreshing then exit;
    Refreshing := True;   // I am busy now.

    LV.Items.BeginUpdate;
    try
        SL:=TStringList.Create;
        try
            GetProcessList(SL); // It is key call. Fills the SL with names and PIDs of running processes

            // Let's update the list

            // Add new processes
            for i:=0 to SL.Count-1 do
            begin
                PID:=Integer(SL.Objects[i]); // Objects property contains PID
                SPID:=IntToStr(PID);

                if LV.FindCaption(0,SPID,False,True,True) <> nil then continue; // Process already listed

                // If not the add it to list
                LI:=LV.Items.Add;
                LI.Caption:=SPID;
                LI.SubItems.Add(SL[i]);
            end;

            // Remove terminated processes
            for i:=LV.Items.Count-1 downto 0 do
            begin
                LI:=LV.Items[i];
                PID:=StrToInt(LI.Caption);
                if SL.IndexOfObject(Pointer(PID)) >= 0 then continue; // Process runs
                LI.Free; // Process terminated - remove it from list
            end;
        finally
            SL.Free;
        end;
        Resort;
    finally
        LV.Items.EndUpdate;
        Refreshing:=False; // We are free again.
    end;
end;

// Sorting callback procedure
function SortByColumn(lParam1, lParam2, lParamSort: Integer): Integer stdcall;
var L1,L2: TListItem;
    PID1,PID2: Integer;
begin
    result:=0;
    L1:=TListItem(lParam1);
    L2:=TListItem(lParam2);
    case ProcessListForm.SortIdx of
        0:
            begin
                PID1:=StrToInt(L1.Caption);
                PID2:=StrToInt(L2.Caption);
                if PID1 < PID2 then result:=-1 else
                if PID1 > PID2 then result:= 1 else
                result:=0;
            end;
        1: result:=AnsiCompareText(L1.SubItems[0],L2.SubItems[0]);
    else
        exit; // Looks like error
    end;
    if ProcessListForm.Reversed then result:=-result;
end;

procedure TProcessListForm.Resort;
begin
    LV.CustomSort(SortByColumn,0);
end;

procedure TProcessListForm.LVColumnClick(Sender: TObject;
  Column: TListColumn);
begin
    if SortIdx = Column.Index then
    begin
        Reversed:=not Reversed;
    end else
    begin
        SortIdx := Column.Index;
        Reversed:=False;
    end;
    Resort;
end;

procedure TProcessListForm.FormCreate(Sender: TObject);
begin
    BnRefresh.Click; // Collect processes at startup
end;

end.
